@@ -3,6 +3,7 @@ import logging
 from typing import Any, Literal
 from uuid import UUID
 
+from anyio import get_cancelled_exc_class
 from pydantic import BaseModel, Field
 
 from lionagi.protocols.types import ID, Event, EventStatus, IDType, Node
@@ -26,7 +27,7 @@ logger = logging.getLogger("operation")
 
 
 class Operation(Node, Event):
-    operation: BranchOperations
+    operation: BranchOperations | str
     parameters: dict[str, Any] | BaseModel = Field(
         default_factory=dict,
         description="Parameters for the operation",
@@ -74,12 +75,11 @@ class Operation(Node, Event):
         return self.execution.response if self.execution else None
 
     async def invoke(self, branch: Branch):
-        meth = getattr(branch, self.operation, None)
+        meth = branch.get_operation(self.operation)
         if meth is None:
             raise ValueError(f"Unsupported operation type: {self.operation}")
 
         start = asyncio.get_event_loop().time()
-
         try:
             self.execution.status = EventStatus.PROCESSING
             self.branch_id = branch.id
@@ -88,9 +88,9 @@ class Operation(Node, Event):
             self.execution.response = response
             self.execution.status = EventStatus.COMPLETED
 
-        except asyncio.CancelledError:
+        except get_cancelled_exc_class():
             self.execution.error = "Operation cancelled"
-            self.execution.status = EventStatus.FAILED
+            self.execution.status = EventStatus.CANCELLED
             raise
 
         except Exception as e:
