@@ -2,11 +2,16 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 
+import contextlib
+import json
 from enum import Enum
 from typing import Any
 
 from pydantic import Field, field_serializer
 
+from lionagi.ln import Unset
+from lionagi.utils import to_dict
+
 from .element import Element
 
 __all__ = (
@@ -16,6 +21,9 @@ __all__ = (
 )
 
 
+_SIMPLE_TYPE = (str, bytes, bytearray, int, float, type(None), Enum)
+
+
 class EventStatus(str, Enum):
     """Status states for tracking action execution progress.
 
@@ -24,12 +32,14 @@ class EventStatus(str, Enum):
         PROCESSING: Action is currently being executed.
         COMPLETED: Action completed successfully.
         FAILED: Action failed during execution.
+        SKIPPED: Action was skipped due to unmet conditions.
     """
 
     PENDING = "pending"
     PROCESSING = "processing"
     COMPLETED = "completed"
     FAILED = "failed"
+    SKIPPED = "skipped"
     CANCELLED = "cancelled"
     ABORTED = "aborted"
 
@@ -79,6 +89,44 @@ class Execution:
             f"response={self.response}, error={self.error})"
         )
 
+    def to_dict(self) -> dict:
+        """Converts the execution state to a dictionary.
+
+        Returns:
+            dict: A dictionary representation of the execution state.
+        """
+        res_ = Unset
+        json_serializable = True
+
+        if not isinstance(self.response, _SIMPLE_TYPE):
+            json_serializable = False
+            try:
+                # check whether response is JSON serializable
+                json.dumps(self.response)
+                res_ = self.response
+                json_serializable = True
+            except Exception:
+                with contextlib.suppress(Exception):
+                    # attempt to convert to dict
+                    d_ = to_dict(
+                        self.response,
+                        recursive=True,
+                        recursive_python_only=False,
+                    )
+                    json.dumps(d_)
+                    res_ = d_
+                    json_serializable = True
+
+        if res_ is Unset and not json_serializable:
+            res_ = "<unserializable>"
+
+        return {
+            "status": self.status.value,
+            "duration": self.duration,
+            "response": res_ or self.response,
+            "error": self.error,
+        }
+
 
 class Event(Element):
     """Extends Element with an execution state.
@@ -101,12 +149,7 @@ class Event(Element):
             dict: The serialized data containing status, duration, response,
             and error fields.
         """
-        return {
-            "status": val.status.value,
-            "duration": val.duration,
-            "response": val.response,
-            "error": val.error,
-        }
+        return val.to_dict()
 
     @property
     def response(self) -> Any:
