@@ -7,7 +7,7 @@ from collections.abc import Callable
 from typing import Any
 
 import pandas as pd
-from pydantic import Field, JsonValue, model_validator
+from pydantic import Field, JsonValue, PrivateAttr, model_validator
 from typing_extensions import Self
 
 from lionagi.protocols.types import (
@@ -31,9 +31,9 @@ from lionagi.protocols.types import (
 )
 
 from .._errors import ItemNotFoundError
+from ..ln import lcall
 from ..service.imodel import iModel
-from ..utils import lcall
-from .branch import Branch
+from .branch import Branch, OperationManager
 
 
 class Session(Node, Communicatable, Relational):
@@ -56,6 +56,60 @@ class Session(Node, Communicatable, Relational):
         default_factory=MailManager, exclude=True
     )
     name: str = Field(default="Session")
+    user: SenderRecipient | None = None
+    _operation_manager: OperationManager = PrivateAttr(
+        default_factory=OperationManager
+    )
+
+    async def ainclude_branches(self, branches: ID[Branch].ItemSeq):
+        async with self.branches:
+            self.include_branches(branches)
+
+    def include_branches(self, branches: ID[Branch].ItemSeq):
+        def _take_in_branch(branch: Branch):
+            if not branch in self.branches:
+                self.branches.include(branch)
+                self.mail_manager.add_sources(branch)
+
+            branch.user = self.id
+            branch._operation_manager = self._operation_manager
+            if self.default_branch is None:
+                self.default_branch = branch
+
+        branches = [branches] if isinstance(branches, Branch) else branches
+
+        for i in branches:
+            _take_in_branch(i)
+
+    def register_operation(
+        self, operation: str, func: Callable, *, update: bool = False
+    ):
+        self._operation_manager.register(operation, func, update=update)
+
+    def operation(self, name: str = None, *, update: bool = False):
+        """
+        Decorator to automatically register functions as operations.
+
+        Args:
+            name: Operation name. If None, uses the function's __name__.
+            update: Whether to update if operation already exists.
+
+        Usage:
+            @session.operation()
+            async def read_issue():
+                ...
+
+            @session.operation("custom_name")
+            async def some_function():
+                ...
+        """
+
+        def decorator(func: Callable) -> Callable:
+            operation_name = name if name is not None else func.__name__
+            self.register_operation(operation_name, func, update=update)
+            return func
+
+        return decorator
 
     @model_validator(mode="after")
     def _add_mail_sources(self) -> Self:
@@ -64,7 +118,7 @@ class Session(Node, Communicatable, Relational):
         if self.default_branch not in self.branches:
             self.branches.include(self.default_branch)
         if self.branches:
-            self.mail_manager.add_sources(self.branches)
+            self.include_branches(self.branches)
         return self
 
     def _lookup_branch_by_name(self, name: str) -> Branch | None:
@@ -102,7 +156,8 @@ class Session(Node, Communicatable, Relational):
         progress: Progression = None,
         tool_manager: ActionManager = None,
         tools: Tool | Callable | list = None,
-        **kwargs,  # additional branch parameters
+        as_default_branch: bool = False,
+        **kwargs,
     ) -> Branch:
         kwargs["system"] = system
         kwargs["system_sender"] = system_sender
@@ -116,13 +171,9 @@ class Session(Node, Communicatable, Relational):
         kwargs["tools"] = tools
         kwargs = {k: v for k, v in kwargs.items() if v is not None}
 
-        from .branch import Branch
-
         branch = Branch(**kwargs)  # type: ignore
-
-        self.branches.include(branch)
-        self.mail_manager.add_sources(branch)
-        if self.default_branch is None:
+        self.include_branches(branch)
+        if as_default_branch:
             self.default_branch = branch
         return branch
 
@@ -179,7 +230,7 @@ class Session(Node, Communicatable, Relational):
         """
         branch: Branch = self.branches[branch]
         branch_clone = branch.clone(sender=self.id)
-        self.branches.append(branch_clone)
+        self.include_branches(branch_clone)
         return branch_clone
 
     def change_default_branch(self, branch: ID.Ref):
@@ -230,10 +281,11 @@ class Session(Node, Communicatable, Relational):
             lambda x: [
                 i for i in self.branches[x].messages if i not in exclude_flag
             ],
-            sanitize_input=True,
-            flatten=True,
-            unique_input=True,
-            unique_output=True,
+            input_unique=True,
+            input_flatten=True,
+            input_dropna=True,
+            output_flatten=True,
+            output_unique=True,
         )
         return Pile(
             collections=messages, item_type={RoledMessage}, strict_type=False
@@ -269,14 +321,15 @@ class Session(Node, Communicatable, Relational):
                 lcall(
                     to_,
                     lambda x: self.mail_manager.send(ID.get_id(x)),
-                    sanitize_input=True,
-                    unique_input=True,
-                    use_input_values=True,
+                    input_unique=True,
+                    input_flatten=True,
+                    input_dropna=True,
+                    input_use_values=True,
                 )
             except Exception as e:
                 raise ValueError(f"Failed to send mail. Error: {e}")
 
-    async def acollect_send_all(self, receive_all: bool = False):
+    async def acollect_send_all(self, receive_all: bool = True):
         """
         Collect and send mail for all branches, optionally receiving all mail.
 
@@ -286,7 +339,7 @@ class Session(Node, Communicatable, Relational):
         async with self.mail_manager.sources:
             self.collect_send_all(receive_all)
 
-    def collect_send_all(self, receive_all: bool = False):
+    def collect_send_all(self, receive_all: bool = True):
         """
         Collect and send mail for all branches, optionally receiving all mail.
 
@@ -296,7 +349,8 @@ class Session(Node, Communicatable, Relational):
         self.collect()
         self.send()
         if receive_all:
-            lcall(self.branches, lambda x: x.receive_all())
+            for i in self.branches:
+                i.receive_all()
 
     def collect(self, from_: ID.RefSeq = None):
         """
@@ -316,9 +370,10 @@ class Session(Node, Communicatable, Relational):
                 lcall(
                     from_,
                     lambda x: self.mail_manager.collect(ID.get_id(x)),
-                    sanitize_input=True,
-                    unique_input=True,
-                    use_input_values=True,
+                    input_flatten=True,
+                    input_dropna=True,
+                    input_unique=True,
+                    input_use_values=True,
                 )
             except Exception as e:
                 raise ValueError(f"Failed to collect mail. Error: {e}")
@@ -332,6 +387,7 @@ class Session(Node, Communicatable, Relational):
         max_concurrent: int = 5,
         verbose: bool = False,
         default_branch: Branch | ID.Ref | None = None,
+        alcall_params: Any = None,
     ) -> dict[str, Any]:
         """
         Execute a graph-based workflow using multi-branch orchestration.
@@ -346,7 +402,7 @@ class Session(Node, Communicatable, Relational):
             max_concurrent: Maximum concurrent operations (branches)
             verbose: Enable verbose logging
             default_branch: Branch to use as default (defaults to self.default_branch)
-            **kwargs: Additional arguments passed to operations
+            alcall_params: Parameters for async parallel call execution
 
         Returns:
             Execution results with completed operations and final context
@@ -366,6 +422,7 @@ class Session(Node, Communicatable, Relational):
             parallel=parallel,
             max_concurrent=max_concurrent,
             verbose=verbose,
+            alcall_params=alcall_params,
         )
 
 
